#       multiselect.bash
#       This file is part of zenpakt

#       Copyright 2010 Anshul Narula aka sunny <sunny_delhi18@yahoo.com>
#
#       Zenpakt is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.


#########excerpts from old multiselect
#13,36
#case $1 in
  #-c)
    #local headermsg=$2 headercurr=$3 headertot=$4 MAXRM=$6 MAXLIST=$7 msg=$5
    #shift 7
    #;;

  #*)
    #local MAXRM=$2 MAXLIST=$3 msg=$1
    #shift 3
    #;;
#esac

#(((MAXRM)?($# > MAXRM):0)) && return 1     #WE WONT CHECK THIS ANY MORE PLEASE CHECK YOURSELF

#local i=1
#print -error "$msg"
#multiselect=()
#PS3="(no package selected)?"

#if ((MAXRM?MAXRM > MAXLIST:1)) ; then
  #local extraopts=( all "all in the current list" quit  clear next previous done )
#else
  #local extraopts=( all quit clear done )
#fi
###
 #if [[ $headermsg ]] ; then
      #zp_header "$headermsg" "$headercurr" "$headertot"
    #else
      #zp_header
    #fi
####################################
#
# completely rewritten to use a new interface; requires tput; which we would anyways use in  ask mode, where it is to be used
# lets see how it fares out
$DOC
 USAGE: multiselect [-m message][-t title_of_tab1][-T title_of_tab2][-s startline] list..

list is displayed using printf %q format with bash escapings
OPTIONS
   m     message displayed on top along with <enter h for help>; default none
   t title_of_tab1
         title for first tab; default select packages
   T title_of_tab2
         title for tab 2;default selected packages
   s startline
         The line from where to start the interface; default 0

$ENDDOC

$$

multiselect() {
  #set -x
  ###############
  #future options
  # t  title  set title of tabs[0] the default tab
  # T  title  set title of tabs[1] the second tab
  # s  the start line
  # m  the message
  ###############

  ###############
  #local variable definitions
  ###############
  local currenttab tabs currentline startline currentcontent0 currentcontent1 index pagelength totalcontent0 totalcontent1 currentpage totalpage message

  #the initial message
  message=
  ##index of currently set tab,tabs[0] is the default tab view
  currenttab=0
  #the default value of the two tabs title
  tabs=("select packages" "selected packages")
  #the current line
  currentline=0
  #the start line
  startline=0
  #the current page contents of tab 0 and 1
  currentcontent0=()
  currentcontent1=()
  #the total contents of tab0 and tab1
  totalcontent0=()
  totalcontent1=()
  #array containing indices of currently highlighted package/element
  index=(0 0)
  #the number of lines occupied by the elements of any page
  pagelength=0
  #current page and total pages
  currentpage=(1 1)
  totalpage=(1 1)
  ###############

  ###############
  #parse options
  ###############
  OPTIND=0
  while getopts t:T:s:m: REPLY; do
    case $REPLY in
      t)
	tabs[0]=$OPTARG
	;;
      T)
	tabs[1]=$OPTARG
	;;
      s)
       startline=$OPTARG
       ;;
      m)
       message=$OPTARG
       ;;
    esac
  done
  shift $((OPTIND-1))
  ###############
  #functions definitions
  ###############

  #_multiselect_cycletabs_ newtabsindex
  #change the currenttab to reflect the change in tab view;
  #although we have two tabs this function can handle any number of tabs
  _multiselect_cycletabs_() {
    if (($1 <= (${#tabs[@]}-1) && $1 != currenttab));then
      currenttab=$1
    fi
  }

  #_multiselect_printtab_
  #print all tabs highlighting the current tab;
  #although we have two tabs this function can handle any number of tabs
  _multiselect_printtab_() {
    local v=$((${#tabs[@]}*2))                 #this parameter stores the actual characters written on terminal when we print the values of tabs
    for i in "${tabs[@]}" ; do
      ((v+=${#i}))
    done
    #test if v is greater than columns, we need to adjust v and currentline accordingly
    while ((v > COLUMNS )) ; do
      ((v-=COLUMNS, currentline++))
    done
    if ((v)) ; then
      ((v=COLUMNS-v, currentline++))
    fi
    printf "${TERMINFO_CODES[reverse]}%s ${TERMINFO_CODES[RED]} ${TERMINFO_CODES[restore]}${TERMINFO_CODES[reverse]}" \
	   "${tabs[@]:0:currenttab}" "${TERMINFO_CODES[restore]}${TERMINFO_CODES[CYAN]}${TERMINFO_CODES[red]}${tabs[currenttab]}" "${tabs[@]:currenttab+1}"
    printf '%*s%s\n' $v '' "${TERMINFO_CODES[restore]}"
  }

  #_multiselect_printcont_
  #print the contents of current tab as defined by currenttab ; highlighting the current index index package for same using contents from _curr_pgcont[01]_
  _multiselect_printcont_() {
    local c
    case $currenttab in
      0)if ((${#currentcontent0[@]})) ; then
	  ((index[0])) && [[ ${currentcontent0[index[0]-1]} ]] && \
	  printf '%q\n'     "${currentcontent0[@]:0:index[0]}"
	  printf '%s%q%s\n' "${TERMINFO_CODES[BLUE]}" "${currentcontent0[index[0]]}" "${TERMINFO_CODES[restore]}"
	  [[ ${currentcontent0[index[0]+1]} ]] && \
	  printf '%q\n'     "${currentcontent0[@]:index[0]+1}"
	fi
	tput cup $((currentline+pagelength+1)) 0
	printf '\n%s\n'   "${TERMINFO_CODES[reverse]}page:${currentpage[0]}/${totalpage[0]}${TERMINFO_CODES[restore]} ${TERMINFO_CODES[reverse]}Total package(s) in current tab: ${#totalcontent0[@]}${TERMINFO_CODES[restore]}"
	((currentline+=pagelength+2))
	;;
      1)
	if ((${#currentcontent1[@]})) ; then
	  ((index[1])) && \
	  printf '%q\n'      "${currentcontent1[@]:0:index[1]}"
	  printf '%s%q%s\n'  "${TERMINFO_CODES[BLUE]}" "${currentcontent1[index[1]]}" "${TERMINFO_CODES[restore]}"
	  [[ ${currentcontent1[index[1]+1]} ]] && \
	  printf '%q\n'      "${currentcontent1[@]:index[1]+1}"
	fi
	tput cup $((currentline+pagelength+1)) 0
	printf '\n%s\n'      "${TERMINFO_CODES[reverse]}page:${currentpage[1]}/${totalpage[1]}${TERMINFO_CODES[restore]} ${TERMINFO_CODES[reverse]}Total package(s) in current tab: ${#totalcontent1[@]}${TERMINFO_CODES[restore]}"
	((currentline+=pagelength+2))
	;;
    esac
  }

  #print a separate help screen providing a summary of help
  _multiselect_printhelp_() {
    tput cup $startline 0
    tput ed
    printf '%s\n' "${TERMINFO_CODES[CYAN]}help${TERMINFO_CODES[restore]}"
    #((currentline+=2))
    printf '%s\n' "This interface allows you to select multiple packages.There are two view tabs,'${tabs[0]}' and '${tabs[1]}'.The former shows you packages which have not yet been selected while the latter shows you the packages which have been selected already.If a package name contains special characters then the characters are printed in quoted format recognised by BASH."
    printf "${TERMINFO_CODES[BLUE]}%s${TERMINFO_CODES[restore]} ${TERMINFO_CODES[reverse]}%s${TERMINFO_CODES[restore]} ${TERMINFO_CODES[cyan]}| ${TERMINFO_CODES[restore]}" \
	     enter "select/deselect" tab "switch tab" x "done" a "select/deselect all" p "select/deselect all in page" u "up" j "down" space "page down" / "page up" h "help"
    printf \\n
    read -sn 1 -p "press any key to continue"
  }

  #set the contents to be printed: currentcontent0 and currentcontent1
  #set the contents using currentpage values
  _multiselect_setcont_() {
    #the maximum number of lines that we can print now, last two lines are used to display current page/total page
    pagelength=$((LINES-currentline-4))
    local m=$pagelength i
    case $1 in
      0)
	currentcontent0=( "${totalcontent0[@]:(currentpage[0]-1)*pagelength:pagelength}" )
	#for i in "${!currentcontent0[@]}" ; do
	  ##[[ $currentcontent0[i] == *$'\n'* ]] &&
	  #currentcontent0[i]=$(printf '%q ' "${currentcontent0[i]}")  #print it in shell usable format
	#done
	if ((${#totalcontent0[@]} > m)); then
	  (((${#totalcontent0[@]}%m)?(totalpage[0]=${#totalcontent0[@]}/m+1):(totalpage[0]=${#totalcontent0[@]}/m)))
	else
	  totalpage[0]=1
	fi
	;;
      1)
	currentcontent1=( "${totalcontent1[@]:(currentpage[1]-1)*pagelength:pagelength}" )
	#for i in "${!currentcontent1[@]}" ; do
	  ##[[ $currentcontent1[i] == $'\n' ]] &&
	  #currentcontent1[i]=$(printf '%q ' "${currentcontent1[i]}")  #print it in shell usable format
	#done
	if ((${#totalcontent1[@]} > m)) ; then
	  (((${#totalcontent1[@]}%m)?(totalpage[1]=${#totalcontent1[@]}/m+1):(totalpage[1]=${#totalcontent1[@]}/m)))
	else
	  totalpage[1]=1
	fi
	;;
    esac
  }

  # $0 currenttab up|down
  #set the currentpage array
  _multiselect_setcurr_pg_() {
    case $1 in
      up)
	if (((currentpage[$currenttab] != 1)?currentpage[$currenttab]--:0)) ;then
	  index[$currenttab]=0
	fi
	;;
      down)
	if (((currentpage[$currenttab] != totalpage[$currenttab])?currentpage[$currenttab]++:0)) ; then
	  index[$currenttab]=0
	fi
	;;
    esac
  }

  _multiselect_printhelpbar_() {
    printf '%s\n%s' "${TERMINFO_CODES[BLUE]}$message<press h for help>${TERMINFO_CODES[restore]}" \
		  "${TERMINFO_CODES[YELLOW]}${TERMINFO_CODES[red]}${TERMINFO_CODES[bold]}"
    putline -v
    printf '%s' "${TERMINFO_CODES[restore]}"
    ((currentline+=2))
  }

  _multiselect_refresh_() {
    tput cup $1 0
    tput ed
    currentline=$1
    _multiselect_cycletabs_ $2
    _multiselect_printhelpbar_
    _multiselect_printtab_
    _multiselect_setcont_ $2
    _multiselect_printcont_
  }

  _multiselect_sel_des_() {
    local v
    case $currenttab in
      0)
	set -- $(( ((currentpage[0]-1)*pagelength) + $1 )) $1
	if [[ ${totalcontent0[$1]} ]] ; then
	  totalcontent1+=("${totalcontent0[$1]}")
	  totalcontent0=( "${totalcontent0[@]:0:$1}" "${totalcontent0[@]:$1+1}")
	else
	  return
	fi
	if [[ ${totalcontent0[$1]} ]] ; then
	  index[0]=$2
	elif (($2 != 0)) && [[ ${totalcontent0[$1-1]} ]];then
	  ((index[0]=$2-1))
	elif (($1 != 0)) && [[ ${totalcontent0[$1-1]} ]] ; then
	  _multiselect_setcurr_pg_ up
	  index[0]=$((pagelength-1))
	else
	  index[0]=0
	fi
	;;
      1)
	set -- $(( ((currentpage[1]-1)*pagelength) + $2 )) $2
	if [[ ${totalcontent1[$1]} ]] ; then
	totalcontent0+=("${totalcontent1[$1]}")
	totalcontent1=( "${totalcontent1[@]:0:$1}" "${totalcontent1[@]:$1+1}")
	else
	 return
	fi
	if [[ ${totalcontent1[$1]} ]] ; then
	  index[1]=$2
	elif (($2 != 0)) && [[ ${totalcontent1[$1-1]} ]];then
	  ((index[1]=$2-1))
	elif (($1 != 0)) && [[ ${totalcontent0[$1-1]} ]] ; then
	  _multiselect_setcurr_pg_ up
	  index[0]=$((pagelength-1))
	else
	  index[1]=0
	fi
	;;
    esac
  }

  _multiselect_sel_des_all_() {
    case $currenttab in
      0)
	index=(0 0)
	currentpage[0]=1
	totalpage[0]=1
	totalcontent1=("$@")
	totalcontent0=()
	;;
      1)
	index=(0 0)
	totalcontent0=("$@")
	totalcontent1=()
	currentpage[1]=1
	totalpage[1]=1
	;;
    esac
  }

  _multiselect_sel_des_page_() {
    local i
    case $currenttab in
      0)
	index[0]=0
	#if we remove this loop, we have no troubles with repeating sequences/elements because if something is in current page it is not selected yet
	#for i in "${currentcontent0[@]}" ; do
	  #inarray "$i" "${totalcontent1[@]}" >/dev/null 2>&1 || totalcontent1+=("$i")
	#done
	totalcontent1+=("${currentcontent0[@]}")
	totalcontent0=( "${totalcontent0[@]:0:(currentpage-1)*pagelength}" "${totalcontent0[@]:currentpage*pagelength}" )
	if ((currentpage == totalpage));then
	  _multiselect_setcurr_pg_ up
	#elif ((currentpage != 1));then :
	  ##_multiselect_setcurr_pg_ down
	fi
	;;
      1)
	index[1]=0
	#for i in "${currentcontent1[@]}" ; do
	  #inarray "$i" "${totalcontent0[@]}" >/dev/null 2>&1 || totalcontent0+=("$i")
	#done
	totalcontent0+=("${currentcontent1[@]}")
	totalcontent1=( "${totalcontent1[@]:0:(currentpage[1]-1)*pagelength}" "${totalcontent1[@]:currentpage[1]*pagelength}" )
	if ((currentpage[1] == totalpage[1]));then
	  _multiselect_setcurr_pg_ up
	#elif ((currentpage[1] != 1));then
	  #_multiselect_setcurr_pg_ down
	fi
	;;
    esac
  }

  _multiselect_setel_indx_() {
    case $currenttab in
      0)
	if ((!${#currentcontent0[@]})) ; then
	  index[0]=0
	  return
	elif ((index[0] == (${#currentcontent0[@]}-1) )) && [[ $1 = down ]];then
	  ((currentpage == totalpage)) && return
	  _multiselect_setcurr_pg_ $1
	  return
	elif ((!index[0])) && [[ $1 = up ]];then
	  ((currentpage == 1)) && return
	  _multiselect_setcurr_pg_ $1
	  index[0]=$((pagelength-1))
	  return
	fi
	case $1 in
	  up)
	    if ((index[0])) && [[ ${currentcontent0[index[0]-1]} ]];then
	      ((index[0]?(index[0]--):1))
	    fi
	    ;;
	  down)
	    if [[ ${currentcontent0[index[0]+1]} ]] ;then
	      ((index[0]++))
	    fi
	    ;;
	esac
	;;

      1)
	if ((!${#currentcontent1[@]})) ; then
	  index[1]=0
	  return
	elif { ((index[1] == (${#currentcontent1[@]}-1) )) && [[ $1 = down ]] ;} || { ((!index[1])) && [[ $1 = up ]] ;};then
	  _multiselect_setcurr_pg_ $1
	  [[ $1 = up ]] && index[1]=$((pagelength-1))
	  return
	fi
	#selected[current_selected_i]=${selected[current_selected_i]#$b}
	#selected[current_selected_i]=${selected[current_selected_i]%$r}
	case $1 in
	  up)
	    if ((index[1])) && [[ ${currentcontent1[index[1]-1]} ]] ; then
	      ((index[1]?(index[1]--):1))
	    fi
	    ;;
	  down)
	    if [[ ${currentcontent1[index[1]+1]} ]] ;then
	      ((index[1]++))
	    fi
	    ;;
	esac
	#selected[current_selected_i]=$b${selected[current_selected_i]}$r
	;;
    esac
  }

  _multiselect_read_act_() {
    #$1 = startline ; rest is all the stuffs
    #enter select the current highlighted package or unselect it, depends on the view tab you are in
    #tab            change current view tab
    #x              done
    #a              select all,removeall
    #p              select all in current page
    #u              move one package up
    #j              move one package down
    #h              display help message
    #backspace      page up
    local l=$LINES c=$COLUMNS
    while : ; do
      read -sn 1
      case $REPLY in
	$'\t')
	 _multiselect_refresh_ $1 $((!currenttab))
	 ;;

	x)
	 break
	 ;;

	a)
	 _multiselect_sel_des_all_ "${@:2}"
	 _multiselect_refresh_ "$1" "$currenttab"
	 ;;

	p)
	_multiselect_sel_des_page_
	_multiselect_refresh_ "$1" "$currenttab"
	;;
	u)
	  _multiselect_setel_indx_ up
	  _multiselect_refresh_ "$1" "$currenttab"
	  ;;

	j)
	  _multiselect_setel_indx_ down
	  _multiselect_refresh_ "$1" "$currenttab"
	  ;;

	h)
	  _multiselect_printhelp_ $1
	  _multiselect_refresh_ $1 $currenttab
	  ;;

	" ")
	  _multiselect_setcurr_pg_ down
	  _multiselect_refresh_ $1 $currenttab
	  ;;

	/)
	  _multiselect_setcurr_pg_ up
	  _multiselect_refresh_ $1 $currenttab
	  ;;

	 '')
	  _multiselect_sel_des_ "${index[@]}"
	  _multiselect_refresh_ "$1" "$currenttab"
	  ;;

	 *)
	  (($l != $LINES || $c != $COLUMNS)) || continue
	  l=$LINES c=$COLUMNS
	  _multiselect_refresh_ "$1" "$currenttab"
	  ;;
      esac
    done
  }


  totalcontent0=("$@" )
  #lets print our tabs
  _multiselect_refresh_ "$startline" "$currenttab"
  _multiselect_read_act_ $startline "$@"
  multiselect=("${totalcontent1[@]}")
}
